@@ -2,7 +2,7 @@
 //
 //    This file is part of Dire Wolf, an amateur radio packet TNC.
 //
-//    Copyright (C) 2011, 2012, 2013, 2014, 2015  John Langner, WB2OSZ
+//    Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016  John Langner, WB2OSZ
 //
 //    This program is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@
 
 // #define X 1
 
+#include "direwolf.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -79,6 +80,7 @@
 #include "hdlc_rec2.h"
 #include "dlq.h"
 #include "ptt.h"
+#include "dtime_now.h"
 
 
 
@@ -178,7 +180,10 @@ int main (int argc, char *argv[])
 	int err;
 	int c;
 	int channel;
-	time_t start_time;
+
+	double start_time;		// Time when we started so we can measure elapsed time.
+	double duration;		// Length of the audio file in seconds.
+	double elapsed;			// Time it took us to process it.
 
 
 #if defined(EXPERIMENT_G) || defined(EXPERIMENT_H)
@@ -291,35 +296,55 @@ int main (int argc, char *argv[])
             case 'B':				/* -B for data Bit rate */
 						/*    300 implies 1600/1800 AFSK. */
 						/*    1200 implies 1200/2200 AFSK. */
+						/*    2400 implies V.26 */
 						/*    9600 implies scrambled. */
 
               my_audio_config.achan[0].baud = atoi(optarg);
 
               dw_printf ("Data rate set to %d bits / second.\n", my_audio_config.achan[0].baud);
 
-              if (my_audio_config.achan[0].baud < 100 || my_audio_config.achan[0].baud > 10000) {
+              if (my_audio_config.achan[0].baud < MIN_BAUD || my_audio_config.achan[0].baud > MAX_BAUD) {
 		text_color_set(DW_COLOR_ERROR);
-                dw_printf ("Use a more reasonable bit rate in range of 100 - 10000.\n");
+                dw_printf ("Use a more reasonable bit rate in range of %d - %d.\n", MIN_BAUD, MAX_BAUD);
                 exit (EXIT_FAILURE);
               }
+
+	      /* We have similar logic in direwolf.c, config.c, gen_packets.c, and atest.c, */
+	      /* that need to be kept in sync.  Maybe it could be a common function someday. */
+
 	      if (my_audio_config.achan[0].baud < 600) {
                 my_audio_config.achan[0].modem_type = MODEM_AFSK;
                 my_audio_config.achan[0].mark_freq = 1600;
                 my_audio_config.achan[0].space_freq = 1800;
 	        strlcpy (my_audio_config.achan[0].profiles, "D", sizeof(my_audio_config.achan[0].profiles));
 	      }
-	      else if (my_audio_config.achan[0].baud > 2400) {
+	      else if (my_audio_config.achan[0].baud < 1800) {
+                my_audio_config.achan[0].modem_type = MODEM_AFSK;
+                my_audio_config.achan[0].mark_freq = DEFAULT_MARK_FREQ;
+                my_audio_config.achan[0].space_freq = DEFAULT_SPACE_FREQ;
+		// Should default to E+ or something similar later.
+	      }
+	      else if (my_audio_config.achan[0].baud < 3600) {
+                my_audio_config.achan[0].modem_type = MODEM_QPSK;
+                my_audio_config.achan[0].mark_freq = 0;
+                my_audio_config.achan[0].space_freq = 0;
+	        strlcpy (my_audio_config.achan[0].profiles, "", sizeof(my_audio_config.achan[0].profiles));
+                dw_printf ("Using V.26 QPSK rather than AFSK.\n");
+	      }
+	      else if (my_audio_config.achan[0].baud < 7200) {
+                my_audio_config.achan[0].modem_type = MODEM_8PSK;
+                my_audio_config.achan[0].mark_freq = 0;
+                my_audio_config.achan[0].space_freq = 0;
+	        strlcpy (my_audio_config.achan[0].profiles, "", sizeof(my_audio_config.achan[0].profiles));
+                dw_printf ("Using V.27 8PSK rather than AFSK.\n");
+	      }
+	      else {
                 my_audio_config.achan[0].modem_type = MODEM_SCRAMBLE;
                 my_audio_config.achan[0].mark_freq = 0;
                 my_audio_config.achan[0].space_freq = 0;
 	        strlcpy (my_audio_config.achan[0].profiles, " ", sizeof(my_audio_config.achan[0].profiles));	// avoid getting default later.
                 dw_printf ("Using scrambled baseband signal rather than AFSK.\n");
 	      }
-	      else {
-                my_audio_config.achan[0].modem_type = MODEM_AFSK;
-                my_audio_config.achan[0].mark_freq = 1200;
-                my_audio_config.achan[0].space_freq = 2200;
-	      }
               break;
 
 	    case 'P':				/* -P for modem profile. */
@@ -336,7 +361,7 @@ int main (int argc, char *argv[])
 	      if (decimate < 1 || decimate > 8) {
 		text_color_set(DW_COLOR_ERROR);
 		dw_printf ("Unreasonable value for -D.\n");
-		exit (1);
+		exit (EXIT_FAILURE);
 	      }
 	      dw_printf ("Divide audio sample rate by %d\n", decimate);
 	      my_audio_config.achan[0].decimate = decimate;
@@ -349,7 +374,7 @@ int main (int argc, char *argv[])
 	      if (my_audio_config.achan[0].fix_bits < RETRY_NONE || my_audio_config.achan[0].fix_bits >= RETRY_MAX) {
 		text_color_set(DW_COLOR_ERROR);
 		dw_printf ("Invalid Fix Bits level.\n");
-		exit (1);
+		exit (EXIT_FAILURE);
 	      }
 	      break;	
 
@@ -407,11 +432,10 @@ int main (int argc, char *argv[])
 	  text_color_set(DW_COLOR_ERROR);
           dw_printf ("Couldn't open file for read: %s\n", argv[optind]);
 	  //perror ("more info?");
-          exit (1);
+          exit (EXIT_FAILURE);
         }
 
-	start_time = time(NULL);
-
+	start_time = dtime_now();
 
 /*
  * Read the file header.  
@@ -437,12 +461,12 @@ int main (int argc, char *argv[])
 	if (strncmp(chunk.id, "fmt ", 4) != 0) {
 	  text_color_set(DW_COLOR_ERROR);
           dw_printf ("WAV file error: Found \"%4.4s\" where \"fmt \" was expected.\n", chunk.id);
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
 	if (chunk.datasize != 16 && chunk.datasize != 18) {
 	  text_color_set(DW_COLOR_ERROR);
           dw_printf ("WAV file error: Need fmt chunk datasize of 16 or 18.  Found %d.\n", chunk.datasize);
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
 
         err = fread (&format, (size_t)chunk.datasize, (size_t)1, fp);	
@@ -452,12 +476,26 @@ int main (int argc, char *argv[])
 	if (strncmp(wav_data.data, "data", 4) != 0) {
 	  text_color_set(DW_COLOR_ERROR);
           dw_printf ("WAV file error: Found \"%4.4s\" where \"data\" was expected.\n", wav_data.data);
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
 
-	// TODO: Should have proper message, not abort.
-	assert (format.nchannels == 1 || format.nchannels == 2);
-	assert (format.wbitspersample == 8 || format.wbitspersample == 16);
+	if (format.wformattag != 1) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Sorry, I only understand audio format 1 (PCM).  This file has %d.\n", format.wformattag);
+	  exit (EXIT_FAILURE);
+	}
+
+	if (format.nchannels != 1 && format.nchannels != 2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Sorry, I only understand 1 or 2 channels.  This file has %d.\n", format.nchannels);
+	  exit (EXIT_FAILURE);
+	}
+
+	if (format.wbitspersample != 8 && format.wbitspersample != 16) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Sorry, I only understand 8 or 16 bits per sample.  This file has %d.\n", format.wbitspersample);
+	  exit (EXIT_FAILURE);
+	}
 
         my_audio_config.adev[0].samples_per_sec = format.nsamplespersec;
 	my_audio_config.adev[0].bits_per_sample = format.wbitspersample;
@@ -467,12 +505,16 @@ int main (int argc, char *argv[])
 	if (format.nchannels == 2) my_audio_config.achan[1].valid = 1;
 
 	text_color_set(DW_COLOR_INFO);
-	dw_printf ("%d samples per second\n", my_audio_config.adev[0].samples_per_sec);
-	dw_printf ("%d bits per sample\n", my_audio_config.adev[0].bits_per_sample);
-	dw_printf ("%d audio channels\n", my_audio_config.adev[0].num_channels);
-	dw_printf ("%d audio bytes in file\n", (int)(wav_data.datasize));
+	dw_printf ("%d samples per second.  %d bits per sample.  %d audio channels.\n",
+		my_audio_config.adev[0].samples_per_sec,
+		my_audio_config.adev[0].bits_per_sample,
+		my_audio_config.adev[0].num_channels);
+	duration = (double) wav_data.datasize /
+		((my_audio_config.adev[0].bits_per_sample / 8) * my_audio_config.adev[0].num_channels * my_audio_config.adev[0].samples_per_sec);
+	dw_printf ("%d audio bytes in file.  Duration = %.1f seconds.\n",
+		(int)(wav_data.datasize),
+		duration);
 	dw_printf ("Fix Bits level = %d\n", my_audio_config.achan[0].fix_bits);
-
 		
 /*
  * Initialize the AFSK demodulator and HDLC decoder.
@@ -496,8 +538,10 @@ int main (int argc, char *argv[])
 
             audio_sample = demod_get_sample (ACHAN2ADEV(c));
 
-            if (audio_sample >= 256 * 256)
+            if (audio_sample >= 256 * 256) {
                e_o_f = 1;
+	       continue;
+	    }
 
 	    if (c == 0) sample_number++;
 
@@ -527,20 +571,24 @@ int main (int argc, char *argv[])
 	  dw_printf ("%d\n", count[j]);
 	}
 #endif
-	dw_printf ("%d packets decoded in %d seconds.\n", packets_decoded, (int)(time(NULL) - start_time));
+
+
+	elapsed = dtime_now() - start_time;
+
+	dw_printf ("%d packets decoded in %.3f seconds.  %.1f x realtime\n", packets_decoded, elapsed, duration/elapsed);
 
 	if (error_if_less_than != -1 && packets_decoded < error_if_less_than) {
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("\n * * * TEST FAILED: number decoded is less than %d * * * \n", error_if_less_than);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
 	if (error_if_greater_than != -1 && packets_decoded > error_if_greater_than) {
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("\n * * * TEST FAILED: number decoded is greater than %d * * * \n", error_if_greater_than);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
 
-	exit (0);
+	exit (EXIT_SUCCESS);
 }
 
 
@@ -596,7 +644,7 @@ void rdq_append (rrbb_t rrbb)
  * This is called when we have a good frame.
  */
 
-void dlq_append (dlq_type_t type, int chan, int subchan, int slice, packet_t pp, alevel_t alevel, retry_t retries, char *spectrum)
+void dlq_rec_frame (int chan, int subchan, int slice, packet_t pp, alevel_t alevel, retry_t retries, char *spectrum)
 {	
 	
 	char stemp[500];
@@ -695,6 +743,24 @@ void dlq_append (dlq_type_t type, int chan, int subchan, int slice, packet_t pp,
 	ax25_safe_print ((char *)pinfo, info_len, 0);
 	dw_printf ("\n");
 
+#if 1		// temp experiment  	TODO: remove this.
+
+#include "decode_aprs.h"
+#include "log.h"
+
+	if (ax25_is_aprs(pp)) {
+
+	  decode_aprs_t A;
+
+	  decode_aprs (&A, pp, 0);
+
+	  // Temp experiment to see how different systems set the RR bits in the source and destination.
+	  // log_rr_bits (&A, pp);
+
+	}
+#endif
+
+
 	ax25_delete (pp);
 
 } /* end fake dlq_append */
@@ -740,7 +806,7 @@ static void usage (void) {
 	dw_printf ("\n");
 	dw_printf ("        -0     Use channel 0 (left) of stereo audio (default).\n");
 	dw_printf ("        -1     use channel 1 (right) of stereo audio.\n");
-	dw_printf ("        -1     decode both channels of stereo audio.\n");
+	dw_printf ("        -2     decode both channels of stereo audio.\n");
 	dw_printf ("\n");
 	dw_printf ("        wav-file-in is a WAV format audio file.\n");
 	dw_printf ("\n");
