@@ -24,6 +24,10 @@
  *
  * Purpose:	Packet assembler and disasembler.
  *
+ *		This was written when I was only concerned about APRS which
+ *		uses only UI frames.  ax25_pad2.c, added years later, has
+ *		functions for dealing with other types of frames.
+ *
  *   		We can obtain AX.25 packets from different sources:
  *		
  *		(a) from an HDLC frame.
@@ -77,11 +81,24 @@
  *			SSID = substation ID
  *			0 = zero
  *
+ *		The AX.25 spec states that the RR bits should be 11 if not used.
+ *		There are a couple documents talking about possible uses for APRS.
+ *		I'm ignoring them for now.
+ *		http://www.aprs.org/aprs12/preemptive-digipeating.txt
+ *		http://www.aprs.org/aprs12/RR-bits.txt
+ *
+ *		I don't recall why I originally intended to set the source/destination C bits both to 1.
+ *		Reviewing this 5 years later, after spending more time delving into the
+ *		AX.25 spec, I think it should be 1 for destination and 0 for source.
+ *		In practice you see all four combinations being used by APRS stations
+ *		and no one really cares about these two bits.
+ *
  *	The final octet of the Source has the form:
  *
  *		C R R SSID 0, where,
  *
- *			C = command/response = 1
+ *			C = command/response = 1    (originally, now I think it should be 0 for source.)
+ *						(Haven't gone back to check to see what code actually does.)
  *			R R = Reserved = 1 1
  *			SSID = substation ID
  *			0 = zero (or 1 if no repeaters)
@@ -146,16 +163,13 @@
 
 #define AX25_PAD_C		/* this will affect behavior of ax25_pad.h */
 
+#include "direwolf.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <stdio.h>
 #include <ctype.h>
-#ifndef _POSIX_C_SOURCE
-
-#define _POSIX_C_SOURCE 1
-#endif
 
 #include "regex.h"
 
@@ -163,9 +177,9 @@
 char *strtok_r(char *str, const char *delim, char **saveptr);
 #endif
 
-#include "direwolf.h"
-#include "ax25_pad.h"
+
 #include "textcolor.h"
+#include "ax25_pad.h"
 #include "fcs_calc.h"
 
 /*
@@ -235,7 +249,13 @@ packet_t ax25_new (void)
 /*
  * check for memory leak.
  */
-	if (new_count > delete_count + 100) {
+
+// version 1.4 push up the threshold.   We could have considerably more with connected mode.
+
+	//if (new_count > delete_count + 100) {
+	if (new_count > delete_count + 256) {
+
+
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("Report to WB2OSZ - Memory leak for packet objects.  new=%d, delete=%d\n", new_count, delete_count);
 #if AX25MEMDEBUG
@@ -315,9 +335,18 @@ void ax25_delete (packet_t this_p)
  * Purpose:	Parse a frame in human-readable monitoring format and change
  *		to internal representation.
  *
- * Input:	monitor	- "TNC-2" format of a monitored packet.  i.e.
+ * Input:	monitor	- "TNC-2" monitor format for packet.  i.e.
  *				source>dest[,repeater1,repeater2,...]:information
  *
+ *			The information part can have non-printable characters
+ *			in the form of <0xff>.  This will be converted to single
+ *			bytes.  e.g.  <0x0d> is carriage return.
+ *			In version 1.4H we will allow nul characters which means
+ *			we have to maintain a length rather than using strlen().
+ *			I maintain that it violates the spec but want to handle it
+ *			because it does happen and we want to preserve it when
+ *			acting as an IGate rather than corrupting it.
+ *
  *		strict	- True to enforce rules for packets sent over the air.
  *			  False to be more lenient for packets from IGate server.
  *
@@ -349,17 +378,11 @@ packet_t ax25_from_text (char *monitor, int strict)
 	char *pa;
 	char *saveptr;		/* Used with strtok_r because strtok is not thread safe. */
 
-	static int first_time = 1;
-	static regex_t unhex_re;
-	int e;
-	char emsg[100];
-#define MAXMATCH 1
-	regmatch_t match[MAXMATCH];
-	int keep_going;
-	char temp[512];
 	int ssid_temp, heard_temp;
 	char atemp[AX25_MAX_ADDR_LEN];
 
+	char info_part[AX25_MAX_INFO_LEN+1];
+	int info_len;
 
 	packet_t this_p = ax25_new ();
 
@@ -372,58 +395,15 @@ packet_t ax25_from_text (char *monitor, int strict)
 
 	/* Is it possible to have a nul character (zero byte) in the */
 	/* information field of an AX.25 frame? */
-	/* Yes, but it would be difficult in the from-text case. */
+	/* At this point, we have a normal C string. */
+	/* It is possible that will convert <0x00> to a nul character later. */
+	/* There we need to maintain a separate length and not use normal C string functions. */
 
 	strlcpy (stuff, monitor, sizeof(stuff));
 
-/* 
- * Translate hexadecimal values like <0xff> to non-printing characters.
- * MIC-E message type uses 5 different non-printing characters.
- */
-
-	if (first_time) 
-	{
-	  e = regcomp (&unhex_re, "<0x[0-9a-fA-F][0-9a-fA-F]>", 0);
-	  if (e) {
-	    regerror (e, &unhex_re, emsg, sizeof(emsg));
-	    text_color_set(DW_COLOR_ERROR);
-	    dw_printf ("%s:%d: %s\n", __FILE__, __LINE__, emsg);
-	  }
-
-	  first_time = 0;
-	}
-
-#if 0
-	text_color_set(DW_COLOR_DEBUG);
-	dw_printf ("BEFORE: %s\n", stuff);
-	ax25_safe_print (stuff, -1, 0);
-	dw_printf ("\n");
-#endif
-	keep_going = 1;
-	while (keep_going) {
-	  if (regexec (&unhex_re, stuff, MAXMATCH, match, 0) == 0) {
-	    int n;
-	    char *p;
-  
-	    stuff[match[0].rm_so + 5] = '\0';
-	    n = strtol (stuff + match[0].rm_so + 3, &p, 16);
-	    stuff[match[0].rm_so] = n;
-	    strlcpy (temp, stuff + match[0].rm_eo, sizeof(temp));
-	    strlcpy (stuff + match[0].rm_so + 1, temp, sizeof(stuff)-match[0].rm_so-1);
-	  }
-	  else {
-	    keep_going = 0;
-	  }
-	}
-#if 0
-	text_color_set(DW_COLOR_DEBUG);
-	dw_printf ("AFTER:  %s\n", stuff);
-	ax25_safe_print (stuff, -1, 0);
-	dw_printf ("\n");
-#endif
 
 /*
- * Initialize the packet with two addresses and control/pid
+ * Initialize the packet structure with two addresses and control/pid
  * for APRS.
  */
 	memset (this_p->frame_data + AX25_DESTINATION*7, ' ' << 1, 6);
@@ -433,7 +413,7 @@ packet_t ax25_from_text (char *monitor, int strict)
 	this_p->frame_data[AX25_SOURCE*7+6] = SSID_H_MASK | SSID_RR_MASK | SSID_LAST_MASK;
 
 	this_p->frame_data[14] = AX25_UI_FRAME;
-	this_p->frame_data[15] = AX25_NO_LAYER_3;
+	this_p->frame_data[15] = AX25_PID_NO_LAYER_3;
 
 	this_p->frame_len = 7 + 7 + 1 + 1;
 	this_p->num_addr = (-1);
@@ -453,12 +433,6 @@ packet_t ax25_from_text (char *monitor, int strict)
 	*pinfo = '\0';
 	pinfo++;
 
-	if (strlen(pinfo) > AX25_MAX_INFO_LEN) {
-	  text_color_set(DW_COLOR_ERROR);
-	  dw_printf ("Warning: Information part truncated to %d characters.\n", AX25_MAX_INFO_LEN);
-	  pinfo[AX25_MAX_INFO_LEN] = '\0';
-	}
-	
 /*
  * Separate the addresses.
  * Note that source and destination order is swappped.
@@ -515,7 +489,6 @@ packet_t ax25_from_text (char *monitor, int strict)
  */
 	while (( pa = strtok_r (NULL, ",", &saveptr)) != NULL && this_p->num_addr < AX25_MAX_ADDRS ) {
 
-	  //char *last;
 	  int k;
 
 	  k = this_p->num_addr;
@@ -541,11 +514,61 @@ packet_t ax25_from_text (char *monitor, int strict)
 	  }
         }
 
+
+/*
+ * Finally, process the information part.
+ *
+ * Translate hexadecimal values like <0xff> to single bytes.
+ * MIC-E format uses 5 different non-printing characters.
+ * We might want to manually generate UTF-8 characters such as degree.
+ */
+
+//#define DEBUG14H 1
+
+#if DEBUG14H
+	text_color_set(DW_COLOR_DEBUG);
+	dw_printf ("BEFORE: %s\nSAFE:   ", pinfo);
+	ax25_safe_print (pinfo, -1, 0);
+	dw_printf ("\n");
+#endif
+
+	info_len = 0;
+	while (*pinfo != '\0' && info_len < AX25_MAX_INFO_LEN) {
+
+	  if (strlen(pinfo) >= 6 &&
+		pinfo[0] == '<' &&
+		pinfo[1] == '0' &&
+		pinfo[2] == 'x' &&
+		isxdigit(pinfo[3]) &&
+		isxdigit(pinfo[4]) &&
+		pinfo[5] == '>') {
+
+	    char *p;
+
+	    info_part[info_len] = strtol (pinfo + 3, &p, 16);
+	    info_len++;
+	    pinfo += 6;
+	  }
+	  else {
+	    info_part[info_len] = *pinfo;
+	    info_len++;
+	    pinfo++;
+	  }
+	}
+	info_part[info_len] = '\0';
+
+#if DEBUG14H
+	text_color_set(DW_COLOR_DEBUG);
+	dw_printf ("AFTER:  %s\nSAFE:   ", info_part);
+	ax25_safe_print (info_part, info_len, 0);
+	dw_printf ("\n");
+#endif
+
 /*
  * Append the info part.  
  */
-	strlcpy ((char*)(this_p->frame_data+this_p->frame_len), pinfo, sizeof(this_p->frame_data)-this_p->frame_len);
-	this_p->frame_len += strlen(pinfo);
+	memcpy ((char*)(this_p->frame_data+this_p->frame_len), info_part, info_len);
+	this_p->frame_len += info_len;
 
 	return (this_p);
 }
@@ -891,7 +914,10 @@ packet_t ax25_unwrap_third_party (packet_t from_pp)
 
 	(void) ax25_get_info (from_pp, &info_p);
 
-	result_pp = ax25_from_text((char *)info_p + 1, 0);
+	// Want strict because addresses should conform to AX.25 here.
+	// That's not the case for something from an Internet Server.
+
+	result_pp = ax25_from_text((char *)info_p + 1, 1);
 
 	return (result_pp);
 }
@@ -1513,15 +1539,48 @@ int ax25_get_first_not_repeated(packet_t this_p)
 }
 
 
+/*------------------------------------------------------------------------------
+ *
+ * Name:	ax25_get_rr
+ *
+ * Purpose:	Return the two reserved "RR" bits in the specified address field.
+ *
+ * Inputs:	pp	- Packet object.
+ *
+ *		n	- Index of address.   Use the symbols
+ *			  AX25_DESTINATION, AX25_SOURCE, AX25_REPEATER1, etc.
+ *
+ * Returns:	0, 1, 2, or 3.
+ *
+ *------------------------------------------------------------------------------*/
+
+int ax25_get_rr (packet_t this_p, int n)
+{
+
+	assert (this_p->magic1 == MAGIC);
+	assert (this_p->magic2 == MAGIC);
+	assert (n >= 0 && n < this_p->num_addr);
+
+	if (n >= 0 && n < this_p->num_addr) {
+	  return ((this_p->frame_data[n*7+6] & SSID_RR_MASK) >> SSID_RR_SHIFT);
+	}
+	else {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Internal error: ax25_get_rr(%d), num_addr=%d\n", n, this_p->num_addr);
+	  return (0);
+	}
+}
+
+
 /*------------------------------------------------------------------------------
  *
  * Name:	ax25_get_info
  * 
  * Purpose:	Obtain Information part of current packet.
  *
- * Inputs:	None.
+ * Inputs:	this_p	- Packet object pointer.
  *
- * Outputs:	paddr	- Starting address is returned here.
+ * Outputs:	paddr	- Starting address of information part is returned here.
  *
  * Assumption:	ax25_from_text or ax25_from_frame was called first.
  *
@@ -1562,6 +1621,56 @@ int ax25_get_info (packet_t this_p, unsigned char **paddr)
 
 	*paddr = info_ptr;
 	return (info_len);
+
+} /* end ax25_get_info */
+
+
+/*------------------------------------------------------------------------------
+ *
+ * Name:	ax25_cut_at_crlf
+ *
+ * Purpose:	Truncate the information part at the first CR or LF.
+ *		This is used for the RF>IS IGate function.
+ *		CR/LF is used as record separator so we must remove it
+ *		before packaging up packet to sending to server.
+ *
+ * Inputs:	this_p	- Packet object pointer.
+ *
+ * Outputs:	Packet is modified in place.
+ *
+ * Returns:	Number of characters removed from the end.
+ *		0 if not changed.
+ *
+ * Assumption:	ax25_from_text or ax25_from_frame was called first.
+ *
+ *------------------------------------------------------------------------------*/
+
+int ax25_cut_at_crlf (packet_t this_p)
+{
+	unsigned char *info_ptr;
+	int info_len;
+	int j;
+
+
+	assert (this_p->magic1 == MAGIC);
+	assert (this_p->magic2 == MAGIC);
+
+	info_len = ax25_get_info (this_p, &info_ptr);
+
+	// Can't use strchr because there is potential of nul character.
+
+	for (j = 0; j < info_len; j++) {
+
+	  if (info_ptr[j] == '\r' || info_ptr[j] == '\n') {
+
+	    int chop = info_len - j;
+
+	    this_p->frame_len -= chop;
+	    return (chop);
+	  }
+	}
+
+	return (0);
 }
 
 
@@ -1674,6 +1783,25 @@ double ax25_get_release_time (packet_t this_p)
 }
 
 
+/*------------------------------------------------------------------------------
+ *
+ * Name:	ax25_set_modulo
+ *
+ * Purpose:	Set modulo value for I and S frame sequence numbers.
+ *
+ *------------------------------------------------------------------------------*/
+
+void ax25_set_modulo (packet_t this_p, int modulo)
+{
+	assert (this_p->magic1 == MAGIC);
+	assert (this_p->magic2 == MAGIC);
+
+	this_p->modulo = modulo;
+}
+
+
+
+
 
 /*------------------------------------------------------------------
  *
@@ -1745,6 +1873,64 @@ void ax25_format_addrs (packet_t this_p, char *result)
 }
 
 
+/*------------------------------------------------------------------
+ *
+ * Function:	ax25_format_via_path
+ *
+ * Purpose:	Format via path addresses suitable for printing.
+ *
+ * Inputs:	Current packet.
+ *
+ *		result_size	- Number of bytes available for result.
+ *				  We can have up to 8 addresses x 9 characters
+ *				  plus 7 commas, possible *, and nul = 81 minimum.
+ *
+ * Outputs:	result	- Digipeater field addresses combined into a single string of the form:
+ *
+ *				"repeater, repeater ..."
+ *
+ *			An asterisk is displayed after the last digipeater
+ *			with the "H" bit set.  e.g.  If we hear RPT2,
+ *
+ *			RPT1,RPT2*,RPT3
+ *
+ *			No asterisk means the source is being heard directly.
+ *
+ *------------------------------------------------------------------*/
+
+void ax25_format_via_path (packet_t this_p, char *result, size_t result_size)
+{
+	int i;
+	int heard;
+	char stemp[AX25_MAX_ADDR_LEN];
+
+	assert (this_p->magic1 == MAGIC);
+	assert (this_p->magic2 == MAGIC);
+	*result = '\0';
+
+	/* Don't get upset if no addresses.  */
+	/* This will allow packets that do not comply to AX.25 format. */
+
+	if (this_p->num_addr == 0) {
+	  return;
+	}
+
+	heard = ax25_get_heard(this_p);
+
+	for (i=(int)AX25_REPEATER_1; i<this_p->num_addr; i++) {
+	  if (i > (int)AX25_REPEATER_1) {
+	    strlcat (result, ",", result_size);
+	  }
+	  ax25_get_addr_with_ssid (this_p, i, stemp);
+	  strlcat (result, stemp, result_size);
+	  if (i == heard) {
+	    strlcat (result, "*", result_size);
+	  }
+	}
+
+} /* end ax25_format_via_path */
+
+
 /*------------------------------------------------------------------
  *
  * Function:	ax25_pack
@@ -1789,12 +1975,11 @@ int ax25_pack (packet_t this_p, unsigned char result[AX25_MAX_PACKET_LEN])
  *
  * Inputs:	this_p	- pointer to packet object.
  *		
- * 		modulo	- We often need to know this because context is
- *			  required to know if control is 1 or 2 bytes.
- *
  * Outputs:	desc	- Text description such as "I frame" or
  *			  "U frame SABME".   
- *			  Supply 16 bytes to be safe.
+ *			  Supply 40 bytes to be safe.
+ *
+ *		cr	- Command or response?
  *
  *		pf	- P/F - Poll/Final or -1 if not applicable
  *
@@ -1807,17 +1992,20 @@ int ax25_pack (packet_t this_p, unsigned char result[AX25_MAX_PACKET_LEN])
  *------------------------------------------------------------------*/
 
 // TODO: need someway to ensure caller allocated enough space.
-#define DESC_SIZ 32
+// Should pass in as parameter.
+#define DESC_SIZ 40
 
-ax25_frame_type_t ax25_frame_type (packet_t this_p, ax25_modulo_t modulo, char *desc, int *pf, int *nr, int *ns) 
+
+ax25_frame_type_t ax25_frame_type (packet_t this_p, cmdres_t *cr, char *desc, int *pf, int *nr, int *ns) 
 {
 	int c;		// U frames are always one control byte.
-	int c2;		// I & S frames can have second Control byte.
+	int c2 = 0;	// I & S frames can have second Control byte.
 
 	assert (this_p->magic1 == MAGIC);
 	assert (this_p->magic2 == MAGIC);
 
 	strlcpy (desc, "????", DESC_SIZ);
+	*cr = cr_11;
 	*pf = -1;
 	*nr = -1;
 	*ns = -1;
@@ -1827,16 +2015,65 @@ ax25_frame_type_t ax25_frame_type (packet_t this_p, ax25_modulo_t modulo, char *
 	  strlcpy (desc, "Not AX.25", DESC_SIZ);
 	  return (frame_not_AX25);
 	}
-	if (modulo == modulo_128) {
+
+/*
+ * TERRIBLE HACK :-(  for display purposes.
+ *
+ * I and S frames can have 1 or 2 control bytes but there is
+ * no good way to determine this without dipping into the data
+ * link state machine.  Can we guess?
+ *
+ * S frames have no protocol id or information so if there is one
+ * more byte beyond the control field, we could assume there are
+ * two control bytes.
+ *
+ * For I frames, the protocol id will usually be 0xf0.  If we find
+ * that as the first byte of the information field, it is probably
+ * the pid and not part of the information.  Ditto for segments 0x08.
+ * Not fool proof but good enough for troubleshooting text out.
+ *
+ * If we have a link to the peer station, this will be set properly
+ * before it needs to be used for other reasons.
+ *
+ * Setting one of the RR bits (find reference!) is sounding better and better.
+ * It's in common usage so I should lobby to get that in the official protocol spec.
+ */
+
+	if (this_p->modulo == 0 && (c & 3) == 1 && ax25_get_c2(this_p) != -1) {
+	  this_p->modulo = modulo_128;
+	}
+	else if (this_p->modulo == 0 && (c & 1) == 0 && this_p->frame_data[ax25_get_info_offset(this_p)] == 0xF0) {
+	  this_p->modulo = modulo_128;
+	}
+	else if (this_p->modulo == 0 && (c & 1) == 0 && this_p->frame_data[ax25_get_info_offset(this_p)] == 0x08) {	// same for segments
+	  this_p->modulo = modulo_128;
+	}
+
+
+	if (this_p->modulo == modulo_128) {
 	  c2 = ax25_get_c2 (this_p);
 	}
 
+	int dst_c = this_p->frame_data[AX25_DESTINATION * 7 + 6] & SSID_H_MASK;
+	int src_c = this_p->frame_data[AX25_SOURCE * 7 + 6] & SSID_H_MASK;
+
+	char cr_text[8];
+	char pf_text[8];
+
+	if (dst_c) {
+	  if (src_c) { *cr = cr_11;  strcpy(cr_text,"cc=11"); strcpy(pf_text,"p/f"); }
+	  else       { *cr = cr_cmd; strcpy(cr_text,"cmd");   strcpy(pf_text,"p"); }
+	}
+	else {
+	  if (src_c) { *cr = cr_res; strcpy(cr_text,"res");   strcpy(pf_text,"f"); }
+	  else       { *cr = cr_00;  strcpy(cr_text,"cc=00"); strcpy(pf_text,"p/f"); }
+	}
 
 	if ((c & 1) == 0) {
 
 // Information 			rrr p sss 0		or	sssssss 0  rrrrrrr p
 
-	  if (modulo == modulo_128) {
+	  if (this_p->modulo == modulo_128) {
 	    *ns = (c >> 1) & 0x7f;
 	    *pf = c2 & 1;
 	    *nr = (c2 >> 1) & 0x7f;
@@ -1846,14 +2083,16 @@ ax25_frame_type_t ax25_frame_type (packet_t this_p, ax25_modulo_t modulo, char *
 	    *pf = (c >> 4) & 1;
 	    *nr = (c >> 5) & 7;
 	  }
-	  snprintf (desc, DESC_SIZ, "I frame, n(s)= %d, n(r)=%d, p=%d", *ns, *nr, *pf);
+
+	  //snprintf (desc, DESC_SIZ, "I %s, n(s)=%d, n(r)=%d, %s=%d", cr_text, *ns, *nr, pf_text, *pf);
+	  snprintf (desc, DESC_SIZ, "I %s, n(s)=%d, n(r)=%d, %s=%d, pid=0x%02x", cr_text, *ns, *nr, pf_text, *pf, ax25_get_pid(this_p));
 	  return (frame_type_I);
 	}
 	else if ((c & 2) == 0) {
 
 // Supervisory			rrr p/f ss 0 1		or	0000 ss 0 1  rrrrrrr p/f
 
-	  if (modulo == modulo_128) {
+	  if (this_p->modulo == modulo_128) {
 	    *pf = c2 & 1;
 	    *nr = (c2 >> 1) & 0x7f;
 	  }
@@ -1861,12 +2100,13 @@ ax25_frame_type_t ax25_frame_type (packet_t this_p, ax25_modulo_t modulo, char *
 	    *pf = (c >> 4) & 1;
 	    *nr = (c >> 5) & 7;
 	  }
-	  
+
+ 
 	  switch ((c >> 2) & 3) {
-	    case 0: snprintf (desc, DESC_SIZ, "S frame RR, n(r)=%d, p/f=%d", *nr, *pf);   return (frame_type_S_RR);   break;
-	    case 1: snprintf (desc, DESC_SIZ, "S frame RNR, n(r)=%d, p/f=%d", *nr, *pf);  return (frame_type_S_RNR);  break;
-	    case 2: snprintf (desc, DESC_SIZ, "S frame REJ, n(r)=%d, p/f=%d", *nr, *pf);  return (frame_type_S_REJ);  break;
-	    case 3: snprintf (desc, DESC_SIZ, "S frame SREJ, n(r)=%d, p/f=%d", *nr, *pf); return (frame_type_S_SREJ); break;
+	    case 0: snprintf (desc, DESC_SIZ, "RR %s, n(r)=%d, %s=%d", cr_text, *nr, pf_text, *pf);   return (frame_type_S_RR);   break;
+	    case 1: snprintf (desc, DESC_SIZ, "RNR %s, n(r)=%d, %s=%d", cr_text, *nr, pf_text, *pf);  return (frame_type_S_RNR);  break;
+	    case 2: snprintf (desc, DESC_SIZ, "REJ %s, n(r)=%d, %s=%d", cr_text, *nr, pf_text, *pf);  return (frame_type_S_REJ);  break;
+	    case 3: snprintf (desc, DESC_SIZ, "SREJ %s, n(r)=%d, %s=%d", cr_text, *nr, pf_text, *pf); return (frame_type_S_SREJ); break;
 	 } 
 	}
 	else {
@@ -1877,16 +2117,16 @@ ax25_frame_type_t ax25_frame_type (packet_t this_p, ax25_modulo_t modulo, char *
 	  
 	  switch (c & 0xef) {
 	
-	    case 0x6f: snprintf (desc, DESC_SIZ, "U frame SABME, p=%d", *pf);  return (frame_type_U_SABME); break;
-	    case 0x2f: snprintf (desc, DESC_SIZ, "U frame SABM, p=%d", *pf);   return (frame_type_U_SABM);  break;
-	    case 0x43: snprintf (desc, DESC_SIZ, "U frame DISC, p=%d", *pf);   return (frame_type_U_DISC);  break;
-	    case 0x0f: snprintf (desc, DESC_SIZ, "U frame DM, f=%d", *pf);     return (frame_type_U_DM);    break;
-	    case 0x63: snprintf (desc, DESC_SIZ, "U frame UA, f=%d", *pf);     return (frame_type_U_UA);    break;
-	    case 0x87: snprintf (desc, DESC_SIZ, "U frame FRMR, f=%d", *pf);   return (frame_type_U_FRMR);  break;
-	    case 0x03: snprintf (desc, DESC_SIZ, "U frame UI, pf=%d", *pf);    return (frame_type_U_UI);    break;
-	    case 0xaf: snprintf (desc, DESC_SIZ, "U frame XID, pf=%d", *pf);   return (frame_type_U_XID);   break;
-	    case 0xe3: snprintf (desc, DESC_SIZ, "U frame TEST, pf=%d", *pf);  return (frame_type_U_TEST);  break;
-	    default:   snprintf (desc, DESC_SIZ, "U frame ???");               return (frame_type_U);       break;
+	    case 0x6f: snprintf (desc, DESC_SIZ, "SABME %s, %s=%d",	cr_text, pf_text, *pf);  return (frame_type_U_SABME); break;
+	    case 0x2f: snprintf (desc, DESC_SIZ, "SABM %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_SABM);  break;
+	    case 0x43: snprintf (desc, DESC_SIZ, "DISC %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_DISC);  break;
+	    case 0x0f: snprintf (desc, DESC_SIZ, "DM %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_DM);    break;
+	    case 0x63: snprintf (desc, DESC_SIZ, "UA %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_UA);    break;
+	    case 0x87: snprintf (desc, DESC_SIZ, "FRMR %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_FRMR);  break;
+	    case 0x03: snprintf (desc, DESC_SIZ, "UI %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_UI);    break;
+	    case 0xaf: snprintf (desc, DESC_SIZ, "XID %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_XID);   break;
+	    case 0xe3: snprintf (desc, DESC_SIZ, "TEST %s, %s=%d", 	cr_text, pf_text, *pf);  return (frame_type_U_TEST);  break;
+	    default:   snprintf (desc, DESC_SIZ, "U other???");        				 return (frame_type_U);       break;
 	  }
 	}
 
@@ -1897,6 +2137,8 @@ ax25_frame_type_t ax25_frame_type (packet_t this_p, ax25_modulo_t modulo, char *
 
 } /* end ax25_frame_type */
 
+
+
 /*------------------------------------------------------------------
  *
  * Function:	ax25_hex_dump
@@ -1935,6 +2177,7 @@ static void hex_dump (unsigned char *p, int len)
 }
 
 /* Text description of control octet. */
+// FIXME:  this is wrong.  It doesn't handle modulo 128.
 
 // TODO: use ax25_frame_type() instead.
 
@@ -2083,10 +2326,12 @@ int ax25_is_aprs (packet_t this_p)
 	assert (this_p->magic1 == MAGIC);
 	assert (this_p->magic2 == MAGIC);
 
+	if (this_p->frame_len == 0) return(0);
+
 	ctrl = ax25_get_control(this_p);
 	pid = ax25_get_pid(this_p);
 
-	is_aprs = this_p->num_addr >= 2 && ctrl == AX25_UI_FRAME && pid == AX25_NO_LAYER_3;
+	is_aprs = this_p->num_addr >= 2 && ctrl == AX25_UI_FRAME && pid == AX25_PID_NO_LAYER_3;
 
 #if 0 
         text_color_set(DW_COLOR_ERROR);
@@ -2095,6 +2340,41 @@ int ax25_is_aprs (packet_t this_p)
 	return (is_aprs);
 }
 
+
+/*------------------------------------------------------------------
+ *
+ * Function:	ax25_is_null_frame
+ *
+ * Purpose:	Is this packet structure empty?
+ *
+ * Inputs:	this_p	- pointer to packet object.
+ *
+ * Returns:	True if frame data length is 0.
+ *
+ * Description:	This is used when we want to wake up the
+ *		transmit queue processing thread but don't
+ *		want to transmit a frame.
+ *
+ *------------------------------------------------------------------*/
+
+
+int ax25_is_null_frame (packet_t this_p)
+{
+	int is_null;
+
+	assert (this_p->magic1 == MAGIC);
+	assert (this_p->magic2 == MAGIC);
+
+	is_null = this_p->frame_len == 0;
+
+#if 0
+        text_color_set(DW_COLOR_ERROR);
+        dw_printf ("ax25_is_null_frame(): is_null=%d\n", is_null);
+#endif
+	return (is_null);
+}
+
+
 /*------------------------------------------------------------------
  *
  * Function:	ax25_get_control
@@ -2115,6 +2395,8 @@ int ax25_get_control (packet_t this_p)
 	assert (this_p->magic1 == MAGIC);
 	assert (this_p->magic2 == MAGIC);
 
+	if (this_p->frame_len == 0) return(-1);
+
 	if (this_p->num_addr >= 2) {
 	  return (this_p->frame_data[ax25_get_control_offset(this_p)]);
 	}
@@ -2126,10 +2408,19 @@ int ax25_get_c2 (packet_t this_p)
 	assert (this_p->magic1 == MAGIC);
 	assert (this_p->magic2 == MAGIC);
 
+	if (this_p->frame_len == 0) return(-1);
+
 	if (this_p->num_addr >= 2) {
-	  return (this_p->frame_data[ax25_get_control_offset(this_p)+1]);
+	  int offset2 = ax25_get_control_offset(this_p)+1;
+
+	  if (offset2 < this_p->frame_len) {
+	    return (this_p->frame_data[offset2]);
+	  }
+	  else {
+	    return (-1);	/* attempt to go beyond the end of frame. */
+	  }
 	}
-	return (-1);
+	return (-1);		/* not AX.25 */
 }
 
 
@@ -2159,6 +2450,8 @@ int ax25_get_pid (packet_t this_p)
 	// TODO: handle 2 control byte case.
 	// TODO: sanity check: is it I or UI frame?
 
+	if (this_p->frame_len == 0) return(-1);
+
 	if (this_p->num_addr >= 2) {
 	  return (this_p->frame_data[ax25_get_pid_offset(this_p)]);
 	}
@@ -2296,7 +2589,7 @@ unsigned short ax25_m_m_crc (packet_t pp)
  *
  * Inputs:	pstr	- Pointer to string.
  *
- *		len	- Maximum length if not -1.
+ *		len	- Number of bytes.  If < 0 we use strlen().
  *
  *		ascii_only	- Restrict output to only ASCII.
  *				  Normally we allow UTF-8.
@@ -2347,7 +2640,7 @@ void ax25_safe_print (char *pstr, int len, int ascii_only)
 	if (len > MAXSAFE)
 	  len = MAXSAFE;
 
-	while (len > 0 && *pstr != '\0')
+	while (len > 0)
 	{
 	  ch = *((unsigned char *)pstr);
 
@@ -2437,7 +2730,11 @@ int ax25_alevel_to_text (alevel_t alevel, char text[AX25_ALEVEL_TO_TEXT_SIZE])
 
 	  snprintf (text, AX25_ALEVEL_TO_TEXT_SIZE, "%d(%+d/%+d)", alevel.rec, alevel.mark, alevel.space);
 	}
-	else if (alevel.mark == -2 &&  alevel.space == -2) {		/* DTMF */
+	else if (alevel.mark == -1 &&  alevel.space == -1) {		/* PSK - single number. */
+
+	  snprintf (text, AX25_ALEVEL_TO_TEXT_SIZE, "%d", alevel.rec);
+	}
+	else if (alevel.mark == -2 &&  alevel.space == -2) {		/* DTMF - single number. */
 
 	  snprintf (text, AX25_ALEVEL_TO_TEXT_SIZE, "%d", alevel.rec);
 	}
