@@ -23,6 +23,7 @@
  * Name:	digipeater.c
  *
  * Purpose:	Act as an APRS digital repeater.
+ *		Similar cdigipeater.c is for connected mode.
  *
  *
  * Description:	Decide whether the specified packet should
@@ -53,6 +54,7 @@
 
 #define DIGIPEATER_C
 
+#include "direwolf.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -62,7 +64,6 @@
 #include "regex.h"
 #include <sys/unistd.h>
 
-#include "direwolf.h"
 #include "ax25_pad.h"
 #include "digipeater.h"
 #include "textcolor.h"
@@ -74,8 +75,6 @@
 static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, char *mycall_xmit, 
 				regex_t *uidigi, regex_t *uitrace, int to_chan, enum preempt_e preempt, char *type_filter);
 
-//static int filter_by_type (char *source, char *infop, char *type_filter);
-
 
 /*
  * Keep pointer to configuration options.
@@ -87,6 +86,18 @@ static struct audio_s	    *save_audio_config_p;
 static struct digi_config_s *save_digi_config_p;
 
 
+/*
+ * Maintain count of packets digipeated for each combination of from/to channel.
+ */
+
+static int digi_count[MAX_CHANS][MAX_CHANS];
+
+int digipeater_get_count (int from_chan, int to_chan) {
+	return (digi_count[from_chan][to_chan]);
+}
+
+
+
 /*------------------------------------------------------------------------------
  *
  * Name:	digipeater_init
@@ -165,6 +176,7 @@ void digipeater (int from_chan, packet_t pp)
 	      if (result != NULL) {
 		dedupe_remember (pp, to_chan);
 	        tq_append (to_chan, TQ_PRIO_0_HI, result);
+	        digi_count[from_chan][to_chan]++;
 	      }
 	    }
 	  }
@@ -190,6 +202,7 @@ void digipeater (int from_chan, packet_t pp)
 	      if (result != NULL) {
 		dedupe_remember (pp, to_chan);
 	        tq_append (to_chan, TQ_PRIO_1_LO, result);
+	        digi_count[from_chan][to_chan]++;
 	      }
 	    }
 	  }
@@ -243,6 +256,10 @@ void digipeater (int from_chan, packet_t pp)
  *
  *------------------------------------------------------------------------------*/
 
+#define OBSOLETE14 1
+
+
+#ifndef OBSOLETE14
 static char *dest_ssid_path[16] = { 	
 			"",		/* Use VIA path */
 			"WIDE1-1",
@@ -260,11 +277,13 @@ static char *dest_ssid_path[16] = {
 			"WIDE2-2",	/* South */
 			"WIDE2-2",	/* East */
 			"WIDE2-2"  };	/* West */
+#endif
 				  
 
 static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, char *mycall_xmit, 
 				regex_t *alias, regex_t *wide, int to_chan, enum preempt_e preempt, char *filter_str)
 {
+	char source[AX25_MAX_ADDR_LEN];
 	int ssid;
 	int r;
 	char repeater[AX25_MAX_ADDR_LEN];
@@ -274,19 +293,9 @@ static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, ch
 /*
  * First check if filtering has been configured.
  */
-
-
 	if (filter_str != NULL) {
 
-	  if (pfilter(from_chan, to_chan, filter_str, pp) != 1) {
-
-// TODO1.2: take out debug message
-// Actually it turns out to be useful.
-// Maybe add a quiet option to suppress it although no one has complained about it yet.
-//#if DEBUG
-	    text_color_set(DW_COLOR_DEBUG);
-	    dw_printf ("Packet was rejected for digipeating from channel %d to %d by filter: %s\n", from_chan, to_chan, filter_str);
-//#endif
+	  if (pfilter(from_chan, to_chan, filter_str, pp, 1) != 1) {
 	    return(NULL);
 	  }
 	}
@@ -310,11 +319,15 @@ static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, ch
  * Otherwise we don't want to modify the input because this could be called multiple times.
  */
 
+#ifndef OBSOLETE14		// Took it out in 1.4
+
 	if (ax25_get_num_repeaters(pp) == 0 && (ssid = ax25_get_ssid(pp, AX25_DESTINATION)) > 0) {
 	  ax25_set_addr(pp, AX25_REPEATER_1, dest_ssid_path[ssid]);
 	  ax25_set_ssid(pp, AX25_DESTINATION, 0);
 	  /* Continue with general case, below. */
 	}
+#endif
+
 
 /* 
  * Find the first repeater station which doesn't have "has been repeated" set.
@@ -337,10 +350,13 @@ static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, ch
 
 
 /*
- * First check for explicit use of my call.
+ * First check for explicit use of my call, including SSID.
+ * Someone might explicitly specify a particular path for testing purposes.
+ * This will bypass the usual checks for duplicates and my call in the source.
+ *
  * In this case, we don't check the history so it would be possible
  * to have a loop (of limited size) if someone constructed the digipeater paths
- * correctly.
+ * correctly.  I would expect it only for testing purposes.
  */
 	
 	if (strcmp(repeater, mycall_rec) == 0) {
@@ -356,13 +372,24 @@ static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, ch
 	  return (result);
 	}
 
+/*
+ * Don't digipeat my own.  Fixed in 1.4 dev H.
+ * Alternatively we might feed everything transmitted into
+ * dedupe_remember rather than only frames out of digipeater.
+ */
+	ax25_get_addr_with_ssid(pp, AX25_SOURCE, source);
+	if (strcmp(source, mycall_rec) == 0) {
+	  return (NULL);
+	}
+
+
 /*
  * Next try to avoid retransmitting redundant information.
  * Duplicates are detected by comparing only:
  *	- source
  *	- destination
  *	- info part
- *	- but none of the digipeaters
+ *	- but not the via path.  (digipeater addresses)
  * A history is kept for some amount of time, typically 30 seconds.
  * For efficiency, only a checksum, rather than the complete fields
  * might be kept but the result is the same.
@@ -371,7 +398,6 @@ static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, ch
  *
  */
 
-
 	if (dedupe_check(pp, to_chan)) {
 //#if DEBUG
 	  /* Might be useful if people are wondering why */
@@ -385,7 +411,10 @@ static packet_t digipeat_match (int from_chan, packet_t pp, char *mycall_rec, ch
 
 /*
  * For the alias pattern, we unconditionally digipeat it once.
- * i.e.  Just replace it with MYCALL don't even look at the ssid.
+ * i.e.  Just replace it with MYCALL.
+ *
+ * My call should be an implied member of this set.
+ * In this implementation, we already caught it further up.
  */
 	err = regexec(alias,repeater,0,NULL,0);
 	if (err == 0) {
@@ -584,13 +613,11 @@ static int failed;
 
 static enum preempt_e preempt = PREEMPT_OFF;
 
-static char typefilter[20] = "";
 
 
 static void test (char *in, char *out)
 {
 	packet_t pp, result;
-	//int should_repeat;
 	char rec[256];
 	char xmit[256];
 	unsigned char *pinfo;
@@ -610,6 +637,7 @@ static void test (char *in, char *out)
 
 	ax25_format_addrs (pp, rec);
 	info_len = ax25_get_info (pp, &pinfo);
+	(void)info_len;
 	strlcat (rec, (char*)pinfo, sizeof(rec));
 
 	if (strcmp(in, rec) != 0) {
@@ -784,17 +812,22 @@ int main (int argc, char *argv[])
 
 
 /* 
- * Change destination SSID to normal digipeater if none specified.
+ * Change destination SSID to normal digipeater if none specified.  (Obsolete, removed.)
  */
-	test (	"W1ABC>TEST-3:",
-		"W1ABC>TEST,WB2OSZ-9*,WIDE3-2:");
 
+	test (	"W1ABC>TEST-3:",
+#ifndef OBSOLETE14
+		"W1ABC>TEST,WB2OSZ-9*,WIDE3-2:");
+#else
+		"");
+#endif
 	test (	"W1DEF>TEST-3,WIDE2-2:",
 		"W1DEF>TEST-3,WB2OSZ-9*,WIDE2-1:");
 
 /*
  * Drop duplicates within specified time interval.
  * Only the first 1 of 3 should be retransmitted.
+ * The 4th case might be controversial.
  */
 
 	test (	"W1XYZ>TEST,R1*,WIDE3-2:info1",
@@ -806,6 +839,10 @@ int main (int argc, char *argv[])
 	test (	"W1XYZ>TEST,R3*,WIDE3-2:info1",
 		"");
 
+	test (	"W1XYZ>TEST,R1*,WB2OSZ-9:has explicit routing",
+		"W1XYZ>TEST,R1,WB2OSZ-9*:has explicit routing");
+
+
 /*
  * Allow same thing after adequate time.
  */
@@ -863,73 +900,25 @@ int main (int argc, char *argv[])
 		"");
 
 
-#if 0	/* changed strategy */
-/*
- * New in version 1.2.
- */
-
-
-	// no filter.
-	if (filter_by_type ("CWAPID", ":NWS-TTTTT:DDHHMMz,ADVISETYPE,zcs{seq#", "") != 1) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 1\n"); failed++; }
-	
-	// message should not match psqt
-	if (filter_by_type ("CWAPID", ":NWS-TTTTT:DDHHMMz,ADVISETYPE,zcs{seq#", "pqst") != 0) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 2\n"); failed++; }
-	
-	// This should match position
-	if (filter_by_type ("N3LEE-7", "`cHDl <0x1c>[/\"5j}", "qstp") != 1) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 3\n"); failed++; }
-	
-	// This should match nws
-	if (filter_by_type ("CWAPID", ":NWS-TTTTT:DDHHMMz,ADVISETYPE,zcs{seq#", "n") != 1) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 4\n"); failed++; }
-	
-	// But not this.
-	if (filter_by_type ("CWAPID", ":zzz-TTTTT:DDHHMMz,ADVISETYPE,zcs{seq#", "n") != 0) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 5\n"); failed++; }
-
-	// This should match nws
-	if (filter_by_type ("CWAPID", ";CWAttttz *DDHHMMzLATLONICONADVISETYPE{seq#", "n") != 1) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 6\n"); failed++; }
-
-	// But not this due do addressee prefix mismatch
-	if (filter_by_type ("CWAPID", ";NWSttttz *DDHHMMzLATLONICONADVISETYPE{seq#", "n") != 0) 
-	  { text_color_set(DW_COLOR_ERROR); dw_printf ("filter_by_type case 7\n"); failed++; }
-
-
-/*
- * Filtering integrated with rest of process...
- */
-
-	strlcpy (typefilter, "w", sizeof(typefilter));
-
-	test (	"N8VIM>APN391,WIDE2-1:$ULTW00000000010E097D2884FFF389DC000102430002033400000000",
-		"N8VIM>APN391,WB2OSZ-9*:$ULTW00000000010E097D2884FFF389DC000102430002033400000000");
-
-	test (	"AB1OC-10>APWW10,WIDE1-1,WIDE2-1:>FN42er/# Hollis, NH iGate Operational",
-		"");
- 
-	strlcpy (typefilter, "s", sizeof(typefilter));
-
-	test (	"AB1OC-10>APWW10,WIDE1-1,WIDE2-1:>FN42er/# Hollis, NH iGate Operational",
-		"AB1OC-10>APWW10,WB2OSZ-9*,WIDE2-1:>FN42er/# Hollis, NH iGate Operational");
-
-	test (	"K1ABC-9>TR4R8R,WIDE1-1:`c6LlIb>/`\"4K}_%",
-		"");
-
-	strlcpy (typefilter, "up", sizeof(typefilter));
-
-	test (	"K1ABC-9>TR4R8R,WIDE1-1:`c6LlIb>/`\"4K}_%",
-		"K1ABC-9>TR4R8R,WB2OSZ-9*:`c6LlIb>/`\"4K}_%");
-
-	strlcpy (typefilter, "", sizeof(typefilter));
-#endif
-
 /* 
  * Did I miss any cases?
+ * Yes.  Don't retransmit my own.  1.4H
  */
 
+	test (	"WB2OSZ-7>TEST14,WIDE1-1,WIDE1-1:stuff",
+		"WB2OSZ-7>TEST14,WB2OSZ-9*,WIDE1-1:stuff");
+
+	test (	"WB2OSZ-9>TEST14,WIDE1-1,WIDE1-1:from myself",
+		"");
+
+	test (	"WB2OSZ-9>TEST14,WIDE1-1*,WB2OSZ-9:from myself but explicit routing",
+		"WB2OSZ-9>TEST14,WIDE1-1,WB2OSZ-9*:from myself but explicit routing");
+
+	test (	"WB2OSZ-15>TEST14,WIDE1-1,WIDE1-1:stuff",
+		"WB2OSZ-15>TEST14,WB2OSZ-9*,WIDE1-1:stuff");
+
+
+
 	if (failed == 0) {
 	  dw_printf ("SUCCESS -- All digipeater tests passed.\n");
 	}
