@@ -3,7 +3,7 @@
  */
 
 
-int gen_tone_init (struct audio_s *pp, int amp);
+int gen_tone_init (struct audio_s *pp, int amp, int gen_packets);
 
 
 //int gen_tone_open (int nchan, int sample_rate, int bit_rate, int f1, int f2, int amp, char *fname);
