@@ -6,11 +6,14 @@
 
 #include "decode_aprs.h"	// for decode_aprs_t
 
+#include "ax25_pad.h"
 
 
 
 void log_init (char *path);	
 
-void log_write (int chan, decode_aprs_t *A, packet_t pp, int alevel, retry_t retries);
+void log_write (int chan, decode_aprs_t *A, packet_t pp, alevel_t alevel, retry_t retries);
+
+void log_rr_bits (decode_aprs_t *A, packet_t pp);
 
 void log_term (void); 	
\ No newline at end of file
