@@ -4,15 +4,11 @@
 #define RRBB_H
 
 
-/* Try something new in version 1.0 */
-/* Get back to this later.  Disable for now. */
-
-//#define SLICENDICE 1
-
-typedef short slice_t;
+#define FASTER13 1		// Don't pack 8 samples per byte.
 
 
-#ifdef RRBB_C
+//typedef short slice_t;
+
 
 /* 
  * Maximum size (in bytes) of an AX.25 frame including the 2 octet FCS. 
@@ -28,84 +24,69 @@ typedef short slice_t;
 
 #define MAX_NUM_BITS (MAX_FRAME_LEN * 8 * 6 / 5)
 
-#define SOI 32
-
 typedef struct rrbb_s {
 	int magic1;
 	struct rrbb_s* nextp;	/* Next pointer to maintain a queue. */
+
 	int chan;		/* Radio channel from which it was received. */
 	int subchan;		/* Which modem when more than one per channel. */
-	int audio_level;	/* Received audio level at time of frame capture. */
-	unsigned int len;	/* Current number of samples in array. */
-	int fix_bits;		/* Level of effort to recover from */
-				/* a bad FCS on the frame. */
+	int slice;		/* Which slicer. */
 
+	alevel_t alevel;	/* Received audio level at time of frame capture. */
+	unsigned int len;	/* Current number of samples in array. */
 
 	int is_scrambled;	/* Is data scrambled G3RUH / K9NG style? */
 	int descram_state;	/* Descrambler state before first data bit of frame. */
+	int prev_descram;	/* Previous descrambled bit. */
+
+	unsigned char fdata[MAX_NUM_BITS];
 
-#if SLICENDICE
-	slice_t slice_val;
-	slice_t data[MAX_NUM_BITS];
-#else
-	unsigned int data[(MAX_NUM_BITS+SOI-1)/SOI];
-	unsigned int computed_data[MAX_NUM_BITS];
-#endif
 	int magic2;
 } *rrbb_t;
 
-#else
-
-/* Hide the implementation. */
-
-typedef void *rrbb_t;
-
-#endif
 
 
+rrbb_t rrbb_new (int chan, int subchan, int slice, int is_scrambled, int descram_state, int prev_descram);
 
-rrbb_t rrbb_new (int chan, int subchan, int is_scrambled, int descram_state);
+void rrbb_clear (rrbb_t b, int is_scrambled, int descram_state, int prev_descram);
 
-void rrbb_clear (rrbb_t b, int is_scrambled, int descram_state);
 
-#if SLICENDICE
-void rrbb2_append_bit (rrbb_t b, float val);
-#else
-void rrbb_append_bit (rrbb_t b, int val);
-#endif
+static inline /*__attribute__((always_inline))*/ void rrbb_append_bit (rrbb_t b, const unsigned char val)
+{
+	if (b->len >= MAX_NUM_BITS) {
+	  return;	/* Silently discard if full. */
+	}
+	b->fdata[b->len] = val;
+	b->len++;
+}
+
+static inline /*__attribute__((always_inline))*/ unsigned char rrbb_get_bit (const rrbb_t b, const int ind)
+{
+	return (b->fdata[ind]);
+}
+
 
 void rrbb_chop8 (rrbb_t b);
 
 int rrbb_get_len (rrbb_t b);
 
-#if SLICENDICE
-void rrbb_set_slice_val (rrbb_t b, slice_t slice_val);
-#endif
-
-int rrbb_get_bit (rrbb_t b, unsigned int ind);
-unsigned int rrbb_get_computed_bit (rrbb_t b, unsigned int ind);
-int rrbb_compute_bits (rrbb_t b);
-
 //void rrbb_flip_bit (rrbb_t b, unsigned int ind);
 
 void rrbb_delete (rrbb_t b);
 
 void rrbb_set_nextp (rrbb_t b, rrbb_t np);
-
 rrbb_t rrbb_get_nextp (rrbb_t b);
 
 int rrbb_get_chan (rrbb_t b);
 int rrbb_get_subchan (rrbb_t b);
+int rrbb_get_slice (rrbb_t b);
 
-void rrbb_set_audio_level (rrbb_t b, int a);
-
-int rrbb_get_audio_level (rrbb_t b);
+void rrbb_set_audio_level (rrbb_t b, alevel_t alevel);
+alevel_t rrbb_get_audio_level (rrbb_t b);
 
 int rrbb_get_is_scrambled (rrbb_t b);
-
 int rrbb_get_descram_state (rrbb_t b);
+int rrbb_get_prev_descram (rrbb_t b);
 
-int rrbb_get_fix_bits(rrbb_t b);
-void rrbb_set_fix_bits(rrbb_t b, int fix_bits);
 
 #endif
